'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.generateRequiredProperties = generateRequiredProperties;
exports.addPrefixedProperties = addPrefixedProperties;
exports.caplitalizeString = caplitalizeString;
exports.generatePrefixedProperty = generatePrefixedProperty;
exports.isPrefixProperty = isPrefixProperty;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _data = require('./data');

var _data2 = _interopRequireDefault(_data);

var _utilsCheckBrowser = require('./utils/checkBrowser');

var _utilsCheckBrowser2 = _interopRequireDefault(_utilsCheckBrowser);

var vendorPrefixes = {
	firefox: 'Moz',
	chrome: 'Webkit',
	safari: 'Webkit',
	ie: 'ms',
	opera: 'O'
};

var prefix = '';
var info = undefined;
var requiredProperties = [];
var ua = typeof navigator !== 'undefined' ? navigator.userAgent : undefined;
var generated = false;

exports['default'] = {
	/**
  * Sets the used userAgent
  * @param {string} userAgent - a valid userAgent string
  */
	setUserAgent: function setUserAgent(userAgent) {
		ua = userAgent;
		generateRequiredProperties();
	},

	/**
  * Returns the currently used userAgent
  */
	getUserAgent: function getUserAgent() {
		return ua;
	},

	/**
  * Processes an object of styles using userAgent specific 
  * @param {Object} styles - Styles object that gets prefixed
  * @param {Boolean} hacks - If hacks should be used to resolve browser differences
  */
	process: function process(styles) {
		var hacks = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

		if (!generated) {
			generateRequiredProperties();
		}
		if (requiredProperties.length > 0) {
			addPrefixedProperties(styles);
		}
		return styles;
	}
};

/**
 * Generates an array of all relevant properties according to the userAgent
 * @param {string} userAgent - a valid userAgent string
 */

function generateRequiredProperties() {
	if (ua) {
		info = (0, _utilsCheckBrowser2['default'])(ua);
		var data = _data2['default'][info.browser];

		//only generate if there is browser data provided
		if (data) {
			prefix = vendorPrefixes[info.browser];

			var propperty = undefined;
			for (propperty in data) {
				if (data[propperty] >= info.version) {
					requiredProperties.push(propperty);
				}
			}
			generated = true;
			return requiredProperties;
		} else {
			console.warn('Your browser seems to not be supported by inline-style-prefixer.');
			console.warn('Please create an issue at https://github.com/rofrischmann/inline-style-prefixer');
			return false;
		}
	} else {
		console.warn('userAgent needs to be set first. Use `.setUserAgent(userAgent)`');
		return false;
	}
}

/**
 * Adds prefixed properties to a style object
 * @param {Map} styles - Style object that gets prefixed properties added
 */

function addPrefixedProperties(styles) {
	var property = undefined;

	for (property in styles) {
		var value = styles[property];

		if (value instanceof Object) {
			addPrefixedProperties(value);
		} else {
			if (isPrefixProperty(property)) {
				styles[generatePrefixedProperty(property)] = value;
			}
		}
	}
	return styles;
}

/**
 * Capitalizes first letter of a string
 * @param {String} str - str to caplitalize
 */

function caplitalizeString(str) {
	return str.charAt(0).toUpperCase() + str.slice(1);
}

/**
 * Returns a prefixed style property
 * @param {String} property - a style property in camelCase
 * @param {String} prefix - evaluated vendor prefix that will be added
 */

function generatePrefixedProperty(property) {
	return prefix + caplitalizeString(property);
}

/**
 * Checks if a property needs to be prefixed
 * @param {String} property - a style property
 */

function isPrefixProperty(property) {
	return requiredProperties.indexOf(property) > -1;
}